local addonName, addon = ...

local config = addon.config
local panel = addon.panel
local L = addon.language
--标题
local title = panel:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLargeLeft')
title:SetPoint('TOPLEFT', 16, -16)
title:SetText(addonName.." v"..config.version)
--将设置面板加到系统插件设置菜单
InterfaceOptions_AddCategory(panel)
--所有组件表
panel.controls = {}
--所有音效列表
local soundList = {
	'', '关',
	'Sound\\Doodad\\BellTollAlliance.ogg', 'Bell 1',
	'Sound\\Doodad\\BellTollHorde.ogg', 'Bell 2',
	'Sound\\Doodad\\FX_Ice_Shard_Impact_03.OGG', 'Ice Shard',
	'Sound\\Doodad\\FX_MK_IMPACT_MED_03.OGG', 'Impact',
	'Sound\\Doodad\\BE_ScryingOrb_Explode.ogg', 'Orb Explode',
	'Sound\\SPELLS\\PetCall.ogg', 'Pet Call',
	'Sound\\character\\PlayerRoars\\CharacterRoarsDwarfMale.ogg', 'Roar Dwarf',
	'Sound\\character\\PlayerRoars\\CharacterRoarsOrcMale.ogg', 'Roar Orc',
	'Sound\\Doodad\\FX_GEN_KUNGFU_02.OGG', 'Swoosh',
	'Sound\\character\\EmoteCatCallWhistle05.ogg', 'Whistle',
	-- Custom sound files user can add to addon folder
	'Interface\\AddOns\\' .. addonName .. '\\sound1.mp3', 'Custom sound1.mp3',
	'Interface\\AddOns\\' .. addonName .. '\\sound2.mp3', 'Custom sound2.mp3',
	'Interface\\AddOns\\' .. addonName .. '\\sound1.ogg', 'Custom sound1.ogg',
	'Interface\\AddOns\\' .. addonName .. '\\sound2.ogg', 'Custom sound2.ogg',
}
--创建唯一命名函数
local UniqueName
do
	local controlID = 1

	function UniqueName(name)
		controlID = controlID + 1
		return string.format('%s_%s_%02d', addonName, name, controlID)
	end
end
--设置面板确定函数
function panel:ConfigOkay()
	for _, control in pairs(self.controls) do
		control.SaveValue(control.currentValue)
	end
end
--设置面板回到默认设置函数
function panel:ConfigDefault()
	for _, control in pairs(self.controls) do
		control.currentValue = control.defaultValue
		control.SaveValue(control.currentValue)
	end
end
--设置面板刷新函数
function panel:ConfigRefresh()
	for _, control in pairs(self.controls) do
		control.currentValue = control.LoadValue()
		control:UpdateValue()
	end
end
--创建选择框函数
function panel:CreateCheckBox(text, LoadValue, SaveValue, defaultValue)
	local checkBox = CreateFrame('CheckButton', UniqueName('CheckButton'), self, 'InterfaceOptionsCheckButtonTemplate')

	checkBox.LoadValue = LoadValue
	checkBox.SaveValue = SaveValue
	checkBox.defaultValue = defaultValue
	checkBox.UpdateValue = function(self) self:SetChecked(self.currentValue) end
	getglobal(checkBox:GetName() .. 'Text'):SetText(text)
	checkBox:SetScript('OnClick', function(self) self.currentValue = self:GetChecked() end)

	self.controls[checkBox:GetName()] = checkBox
	return checkBox
end
--下拉菜单点击函数
local function DropDownOnClick(_, dropDown, selectedValue)
	dropDown.currentValue = selectedValue
	UIDropDownMenu_SetText(dropDown, dropDown.valueTexts[selectedValue])
end
--下拉菜单初始化函数
local function DropDownInitialize(frame)
	local info = UIDropDownMenu_CreateInfo()

	for i=1,#frame.valueList,2 do
		local k, v = frame.valueList[i], frame.valueList[i + 1]
		info.text = v
		info.value = k
		info.checked = frame.currentValue == k
		info.func = DropDownOnClick
		info.arg1, info.arg2 = frame, k
		UIDropDownMenu_AddButton(info)
	end
end
--创建下拉菜单函数
function panel:CreateDropDown(text, valueList, LoadValue, SaveValue, defaultValue)
	local dropDown = CreateFrame('Frame', UniqueName('DropDown'), self, 'UIDropDownMenuTemplate')

	local title = self:CreateFontString(nil, 'ARTWORK', 'GameFontNormalSmallLeft')
	title:SetText(text)
	title:SetPoint('BOTTOMLEFT', dropDown, 'TOPLEFT', 16, 0)

	dropDown.LoadValue = LoadValue
	dropDown.SaveValue = SaveValue
	dropDown.defaultValue = defaultValue
	dropDown.UpdateValue = function(self)
		UIDropDownMenu_SetText(self, self.valueTexts[self.currentValue])
	end

	dropDown.valueList = valueList
	dropDown.valueTexts = {}
	for i=1,#valueList,2 do
		local k, v = valueList[i], valueList[i + 1]
		dropDown.valueTexts[k] = v
	end

	dropDown:SetScript('OnShow', function(self)
		UIDropDownMenu_Initialize(self, DropDownInitialize)
	end)

	UIDropDownMenu_JustifyText(dropDown, 'LEFT')
	UIDropDownMenu_SetWidth(dropDown, 120)
	UIDropDownMenu_SetButtonWidth(dropDown, 144)

	self.controls[dropDown:GetName()] = dropDown
	return dropDown
end
--设置面板初始化函数
function panel:Initialize()
	--添加窗体动作函数
	self.okay = self.ConfigOkay
	self.default = self.ConfigDefault
	self.refresh = self.ConfigRefresh
	--自己打断音效菜单
	local controlDropDownSoundOwn = self:CreateDropDown(
		L["soundOwn"],
		soundList,
		function() return config.soundOwn end,
		function(v) config.soundOwn = v end,
		'')
	controlDropDownSoundOwn:SetPoint('TOPLEFT', title, 'BOTTOMLEFT', 0, -24)

	local controlButtonSoundOwn = CreateFrame('Button', nil, panel)
	controlButtonSoundOwn:SetWidth(16)
	controlButtonSoundOwn:SetHeight(16)
	controlButtonSoundOwn:SetPoint('LEFT', controlDropDownSoundOwn, 'RIGHT', 0, 0)
	controlButtonSoundOwn:SetScript('OnClick', function()
		PlaySoundFile(controlDropDownSoundOwn.currentValue or '', 'MASTER')
	end)
	local speakerOwnOff = controlButtonSoundOwn:CreateTexture(nil, 'BACKGROUND')
	speakerOwnOff:SetTexture('Interface\\Common\\VoiceChat-Speaker')
	speakerOwnOff:SetAllPoints(controlButtonSoundOwn)
	local speakerOwnOn = controlButtonSoundOwn:CreateTexture(nil, 'HIGHLIGHT')
	speakerOwnOn:SetTexture('Interface\\Common\\VoiceChat-On')
	speakerOwnOn:SetAllPoints(controlButtonSoundOwn)
	--其他人打断音效菜单
	local controlDropDownSoundOther = self:CreateDropDown(
		L["soundOther"],
		soundList,
		function() return config.soundOther end,
		function(v) config.soundOther = v end,
		'')
	controlDropDownSoundOther:SetPoint('TOPLEFT', controlDropDownSoundOwn, 'TOPRIGHT', 32, 0)

	local controlButtonSoundOther = CreateFrame('Button', nil, panel)
	controlButtonSoundOther:SetWidth(16)
	controlButtonSoundOther:SetHeight(16)
	controlButtonSoundOther:SetPoint('LEFT', controlDropDownSoundOther, 'RIGHT', 0, 0)
	controlButtonSoundOther:SetScript('OnClick', function()
		PlaySoundFile(controlDropDownSoundOther.currentValue or '', 'MASTER')
	end)
	local speakerOtherOff = controlButtonSoundOther:CreateTexture(nil, 'BACKGROUND')
	speakerOtherOff:SetTexture('Interface\\Common\\VoiceChat-Speaker')
	speakerOtherOff:SetAllPoints(controlButtonSoundOther)
	local speakerOtherOn = controlButtonSoundOther:CreateTexture(nil, 'HIGHLIGHT')
	speakerOtherOn:SetTexture('Interface\\Common\\VoiceChat-On')
	speakerOtherOn:SetAllPoints(controlButtonSoundOther)
	--独自一人时喊话设置
	local controlDropDownSoloOwn = self:CreateDropDown(
		L["soloOwn"],
		{ 'off', L["off"], 'self', L["self"], 'say', L["say"] },
		function() return config.solo.own end,
		function(v) config.solo.own = v end,
		'say')
	controlDropDownSoloOwn:SetPoint('TOPLEFT', controlDropDownSoundOwn, 'BOTTOMLEFT', 0, -24)
	--在副本队伍中喊话设置
	local controlDropDownInstanceOwn = self:CreateDropDown(
		L["instanceOwn"],
		{ 'off', L["off"], 'self', L["self"], 'say', L["say"],  'instance', L["instance"] },
		function() return config.instance.own end,
		function(v) config.instance.own = v end,
		'self')
	controlDropDownInstanceOwn:SetPoint('TOPLEFT', controlDropDownSoloOwn, 'BOTTOMLEFT', 0, -24)

	local controlDropDownInstanceOther = self:CreateDropDown(
		L["instanceOther"],
		{ 'off', L["off"], 'self', L["self"], 'say', L["say"],  'instance', L["instance"] },
		function() return config.instance.other end,
		function(v) config.instance.other = v end,
		'self')
	controlDropDownInstanceOther:SetPoint('TOPLEFT', controlDropDownInstanceOwn, 'TOPRIGHT', 32, 0)
	--在小队中喊话设置
	local controlDropDownGroupOwn = self:CreateDropDown(
		L["partyOwn"],
		{ 'off', L["off"], 'self', L["self"], 'say', L["say"], 'party', L["party"] },
		function() return config.party.own end,
		function(v) config.party.own = v end,
		'self')
	controlDropDownGroupOwn:SetPoint('TOPLEFT', controlDropDownInstanceOwn, 'BOTTOMLEFT', 0, -24)

	local controlDropDownGroupOther = self:CreateDropDown(
		L["partyOther"],
		{ 'off', L["off"], 'self', L["self"], 'say', L["say"], 'party', L["party"] },
		function() return config.party.other end,
		function(v) config.party.other = v end,
		'self')
	controlDropDownGroupOther:SetPoint('TOPLEFT', controlDropDownGroupOwn, 'TOPRIGHT', 32, 0)
	--在团队中喊话设置
	local controlDropDownRaidOwn = self:CreateDropDown(
		L["raidOwn"],
		{ 'off', L["off"], 'self', L["self"], 'say', L["say"], 'party', L["party"], 'raid', L["raid"] },
		function() return config.raid.own end,
		function(v) config.raid.own = v end,
		'say')
	controlDropDownRaidOwn:SetPoint('TOPLEFT', controlDropDownGroupOwn, 'BOTTOMLEFT', 0, -24)

	local controlDropDownRaidOther = self:CreateDropDown(
		L["raidOther"],
		{ 'off', L["off"], 'self', L["self"], 'say', L["say"], 'party', L["party"], 'raid', L["raid"] },
		function() return config.raid.other end,
		function(v) config.raid.other = v end,
		'self')
	controlDropDownRaidOther:SetPoint('TOPLEFT', controlDropDownRaidOwn, 'TOPRIGHT', 32, 0)
	--是否开启自动售卖
	local autoSellEnabled = self:CreateCheckBox(
		L["setAutoSell"],
		function() return config.autoSell end,
		function(v) config.autoSell = v end,
		true)
	autoSellEnabled:SetPoint('TOPLEFT', controlDropDownRaidOwn, 'BOTTOMLEFT', 0, -24)
end

--面板初始化
panel:Initialize()
panel:ConfigRefresh()
panel:Show()





